/*
  Buffered serial for ByVac serial devices
  Copyright (c) 2011 Jim Spence.  All right reserved.
  www.byvac.com - see terms and conditions for using hardware
  
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/  


#include "WConstants.h"
#include "BSerial.h"
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include "pins_arduino.h"

BSerial *BSerial::active_object = 0;

// **************************************************************
// constructor
// **************************************************************
BSerial::BSerial(uint8_t receivePin, uint8_t transmitPin, uint8_t rts_pin)
{
    _receivePin = receivePin;
    _transmitPin = transmitPin;
    _rts_pin = rts_pin;
    uint8_t port = digitalPinToPort(receivePin);
    portInputRegister(port);
    clear_tib();
}
// alternative if no RTS is required
BSerial::BSerial(uint8_t receivePin, uint8_t transmitPin)
{
    _receivePin = receivePin;
    _transmitPin = transmitPin;
    _rts_pin = 0xff;
    clear_tib();
}

// **************************************************************
// initialises serial
// **************************************************************
void BSerial::begin(long speed)
{
    _baudRate = speed;
    _bitPeriod = 1000000 / _baudRate;
    digitalWrite(_transmitPin, HIGH);
    pinMode(_transmitPin, OUTPUT);
    pinMode(_receivePin, INPUT);
    if(_rts_pin != 0xff) {
        pinMode(_rts_pin, OUTPUT);
    }
    activate();
}

// **************************************************************
// Request to send is an output that goes low when the buffer is
// full
// **************************************************************
void BSerial::setRTS(char v)
{
    if(_rts_pin==0xff) return;
    digitalWrite(_rts_pin,v);
    if(v) _bfullF = 0;
    else _bfullF = 1;
}

// **************************************************************
// clears buffer
// **************************************************************
void BSerial::clear_tib()
{
	// buffer reset
	_cib=0;
	_Pptr=_Gptr=0;  // pointer initialise
	setRTS(1);	  // buffer not full
}

// **************************************************************
// input in serial interrupt, this fills a buffer up 
// using pre-incrementing
// **************************************************************
void BSerial::fill_buf()
{
//putch('x');	
	if((_Pptr+1) == _Gptr) setRTS(0); // full 
    // full when trying to wrap on top of Gptr
    if(((_Pptr+1) > TIB_MAX) && _Gptr==0) setRTS(1);
        
    // increment and check for wrap
    if(++_Pptr > TIB_MAX) _Pptr=0;

    // get and place char, also clears IR
    _tib[_Pptr]=read();
    _cib++; // easiest way to keep track
}
			
// **************************************************************
// returns number of characters in the buffer
// **************************************************************
char BSerial::cInbuf()
{
    return _cib;
}

// **************************************************************
// removes a char from the buffer if there is one, if not 
// it will wait here
// **************************************************************
char BSerial::getch (void)
{
	// this point can be used to service non urgent code
	while (!_cib);		// Wait for character
	
   	if(++_Gptr > TIB_MAX) _Gptr=0;
    char c = _tib[_Gptr]; // get char before updating buffer 
    
    // make sure recieve is enabled, buffer not full so room for more
    setRTS(1);
    --_cib;  // redune number of char in buffer
    return c;	
}

// **************************************************************
// Original code from SoftwareSerial
// This now simply gets a charatcer from the digital pin. When
// this is called the pin has already gone low
// **************************************************************
char BSerial::read()
{
  char val = 0;
  int bitDelay = _bitPeriod - clockCyclesToMicroseconds(50);
  
  // one byte of serial data (LSB first)
  // ...--\    /--\/--\/--\/--\/--\/--\/--\/--\/--...
  //	 \--/\--/\--/\--/\--/\--/\--/\--/\--/
  //	start  0   1   2   3   4   5   6   7 stop

  // confirm that this is a real start bit, not line noise
  if (digitalRead(_receivePin) == LOW) {
    // frame start indicated by a falling edge and low start bit
    // jump to the middle of the low start bit
    delayMicroseconds(bitDelay / 2 - clockCyclesToMicroseconds(50));
	
    // offset of the bit in the byte: from 0 (LSB) to 7 (MSB)
    for (int offset = 0; offset < 8; offset++) {
	// jump to middle of next bit
	delayMicroseconds(bitDelay);
	
	// read bit
	val |= digitalRead(_receivePin) << offset;
    }
	
    delayMicroseconds(_bitPeriod);
    
    return val;
  }
  
  return -1;
}

// **************************************************************
// from SoftwareSerial
// **************************************************************
void BSerial::putch(uint8_t b)
{
  int bitDelay = _bitPeriod - clockCyclesToMicroseconds(50); // a digitalWrite is about 50 cycles
  byte mask;

  digitalWrite(_transmitPin, LOW);
  delayMicroseconds(bitDelay);

  for (mask = 0x01; mask; mask <<= 1) {
    if (b & mask){ // choose bit
      digitalWrite(_transmitPin,HIGH); // send 1
    }
    else{
      digitalWrite(_transmitPin,LOW); // send 1
    }
    delayMicroseconds(bitDelay);
  }

  digitalWrite(_transmitPin, HIGH);
  delayMicroseconds(bitDelay);
}

// **************************************************************
// put sting to display with escape codes
// **************************************************************
void BSerial::puts(char *s)
{
    while(*s) 
        putch(*(s++));
}

// **************************************************************
// 
// **************************************************************
inline void BSerial::handle_interrupt()
{
  if (active_object)
  {
    active_object->fill_buf();
  }
}

#if defined(PCINT0_vect)
ISR(PCINT0_vect)
{
  BSerial::handle_interrupt();
}
#endif

#if defined(PCINT1_vect)
ISR(PCINT1_vect)
{
  BSerial::handle_interrupt();
}
#endif

#if defined(PCINT2_vect)
ISR(PCINT2_vect)
{
  BSerial::handle_interrupt();
}
#endif

#if defined(PCINT3_vect)
ISR(PCINT3_vect)
{
  BSerial::handle_interrupt();
}
#endif

// This function sets the current object as the "active"
// one and returns true if it replaces another 
bool BSerial::activate(void)
{
  if (active_object != this)
  {
    cli();
    active_object = this;
    return true;
  }
  return false;
}

// Private Methods /////////////////////////////////////////////////////////////

void BSerial::printNumber(unsigned long n, uint8_t base)
{
  unsigned char buf[8 * sizeof(long)]; // Assumes 8-bit chars. 
  unsigned long i = 0;

  if (n == 0) {
    putch('0');
    return;
  } 

  while (n > 0) {
    buf[i++] = n % base;
    n /= base;
  }

  for (; i > 0; i--)
    putch((char) (buf[i - 1] < 10 ? '0' + buf[i - 1] : 'A' + buf[i - 1] - 10));
}
