/*
  Buffered serial for ByVac serial devices
  Copyright (c) 2011 Jim Spence.  All right reserved.
  www.byvac.com - see terms and conditions for using hardware
  
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/  


#include "WConstants.h"
#include "BVserial.h"
#include <avr/interrupt.h>
#include "pins_arduino.h"


// **************************************************************
// initialises serial
// **************************************************************
void BVserial_init(char rxPin, char txPin, long speed)
{
    _baudRate = speed;
    _receivePin=rxPin;
    _transmitPin=txPin;
    _bitPeriod = 1000000 / _baudRate;
    digitalWrite(_transmitPin, HIGH);
    pinMode(_transmitPin, OUTPUT);
    pinMode(_receivePin, INPUT);
    if(_rts_pin != 0xff) {
        pinMode(_rts_pin, OUTPUT);
    }
}

// **************************************************************
// Request to send is an output that goes low when the buffer is
// full
// **************************************************************
void BVserial_setRTS(char v)
{
    if(_rts_pin==0xff) return;
    digitalWrite(_rts_pin,v);
    if(v) _bfullF = 0;
    else _bfullF = 1;
}

// **************************************************************
// clears buffer
// **************************************************************
void BVserial_flush()
{
	// buffer reset
	_cib=0;
	_Pptr=_Gptr=0;  // pointer initialise
	setRTS(1);	  // buffer not full
}

// **************************************************************
// input in serial interrupt, this fills a buffer up 
// using pre-incrementing
// **************************************************************
void BVserial_fill_buf()
{
//putch('x');	
	if((_Pptr+1) == _Gptr) setRTS(0); // full 
    // full when trying to wrap on top of Gptr
    if(((_Pptr+1) > TIB_MAX) && _Gptr==0) setRTS(1);
        
    // increment and check for wrap
    if(++_Pptr > TIB_MAX) _Pptr=0;

    // get and place char, also clears IR
    _tib[_Pptr]=read();
    _cib++; // easiest way to keep track
}
			
// **************************************************************
// returns number of characters in the buffer
// **************************************************************
char BVserial_cInbuf()
{
    return _cib;
}

// **************************************************************
// removes a char from the buffer if there is one, if not 
// it will wait here
// **************************************************************
char BVserial_getch ()
{
	// this point can be used to service non urgent code
	while (!_cib);		// Wait for character
	
   	if(++_Gptr > TIB_MAX) _Gptr=0;
    char c = _tib[_Gptr]; // get char before updating buffer 
    
    // make sure recieve is enabled, buffer not full so room for more
    setRTS(1);
    --_cib;  // redune number of char in buffer
    return c;	
}

// **************************************************************
// Original code from SoftwareSerial
// This now simply gets a charatcer from the digital pin. When
// this is called the pin has already gone low
// **************************************************************
char BVserial_read()
{
  char val = 0;
  int bitDelay = _bitPeriod - clockCyclesToMicroseconds(50);
  
  // one byte of serial data (LSB first)
  // ...--\    /--\/--\/--\/--\/--\/--\/--\/--\/--...
  //	 \--/\--/\--/\--/\--/\--/\--/\--/\--/
  //	start  0   1   2   3   4   5   6   7 stop

  // confirm that this is a real start bit, not line noise
  if (digitalRead(_receivePin) == LOW) {
    // frame start indicated by a falling edge and low start bit
    // jump to the middle of the low start bit
    delayMicroseconds(bitDelay / 2 - clockCyclesToMicroseconds(50));
	
    // offset of the bit in the byte: from 0 (LSB) to 7 (MSB)
    for (int offset = 0; offset < 8; offset++) {
	// jump to middle of next bit
	delayMicroseconds(bitDelay);
	
	// read bit
	val |= digitalRead(_receivePin) << offset;
    }
	
    delayMicroseconds(_bitPeriod);
    
    return val;
  }
  
  return -1;
}

// **************************************************************
// from SoftwareSerial
// **************************************************************
void BVserial_putch(uint8_t b)
{
  int bitDelay = _bitPeriod - clockCyclesToMicroseconds(50); // a digitalWrite is about 50 cycles
  byte mask;

  digitalWrite(_transmitPin, LOW);
  delayMicroseconds(bitDelay);

  for (mask = 0x01; mask; mask <<= 1) {
    if (b & mask){ // choose bit
      digitalWrite(_transmitPin,HIGH); // send 1
    }
    else{
      digitalWrite(_transmitPin,LOW); // send 1
    }
    delayMicroseconds(bitDelay);
  }

  digitalWrite(_transmitPin, HIGH);
  delayMicroseconds(bitDelay);
}

// **************************************************************
// Interrupt handling of fill buf
// **************************************************************

#if defined(PCINT0_vect)
ISR(PCINT0_vect)
{
  fill_buf();
}
#endif

#if defined(PCINT1_vect)
ISR(PCINT1_vect)
{
  fill_buf();
}
#endif

#if defined(PCINT2_vect)
ISR(PCINT2_vect)
{
  fill_buf();
}
#endif

#if defined(PCINT3_vect)
ISR(PCINT3_vect)
{
  fill_buf();
}
#endif
