/*
  BV4236 RTC and Tempertature sensor from ByVac
  Copyright (c) 2011 Jim Spence.  All right reserved.
  www.byvac.com - see terms and conditions for using hardware
  
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/  

#include "WProgram.h"
#include "bv4236.h"
 
// **************************************************************
// RTC and temperature devices are fixed addresses
// **************************************************************
BV4236::BV4236()
{
  Wire.begin(); // join i2c bus  
}

// ==============================================================
// I2C Temperature sensor section
// ==============================================================
// **************************************************************
// Get temperature reading and return a floating point value
// **************************************************************
double BV4236::getTemp()
{
int high, low;
double val=0;
  Wire.beginTransmission(I2TEMP);
  Wire.send(0); // set correct register
  Wire.endTransmission();
  // value is 16 bits
  Wire.requestFrom(I2TEMP, 2); // returns 2 bytes
  high=Wire.receive(); // high byte, integer value
  low=Wire.receive(); // low byte, decimal value
  Wire.endTransmission();
  if(low & 0x80) val=0.5;
  if(low & 0x40) val+=0.25;
  if(low & 0x20) val+=0.125;
  if(low & 0x10) val+=0.0625;
  val+=high;
  return (val);
}

// **************************************************************
// default is 9 bits, can be set from 9 to 12
// **************************************************************
void BV4236::setResolution(char bits)
{
  if(bits < 9) bits=9;
  if(bits > 12) bits=12;
  Wire.beginTransmission(I2TEMP);
  Wire.send(1); // config register
  bits-=9;
  Wire.send(bits << 5);
  Wire.endTransmission();
}

// ==============================================================

// ==============================================================
// I2C RTC section
// ==============================================================

// **************************************************************
// sets value of time register - converts to BCD
// **************************************************************
void BV4236::setReg(char reg, char value)
{
  Wire.beginTransmission(I2RTC);
  Wire.send(reg); // select register
  Wire.send(dec2bdc(value)); // select register
  Wire.endTransmission();
}

// **************************************************************
// get time register value, this can be any regietser to 
// facilitate getting time values
// returns value converted to decimal 
// **************************************************************
int BV4236::getReg(char reg)
{
int rv;
  Wire.beginTransmission(I2RTC);
  Wire.send(reg); // select register
  Wire.endTransmission();
  Wire.requestFrom(I2RTC, 1); // returns 1 byte
  rv=Wire.receive(); 
  Wire.endTransmission();
  return (bcd2dec(rv));            
}

// **************************************************************
// clears halt bit which is set when powered down
// **************************************************************
void BV4236::clrHalt()
{
char halt;
  Wire.beginTransmission(I2RTC);
  Wire.send(0xc); // select register
  Wire.endTransmission();
  Wire.requestFrom(I2RTC, 1); // returns 1 byte
  halt=Wire.receive(); 
  Wire.endTransmission();
  // clear halt & put back
  halt&=0xaf;
  Wire.beginTransmission(I2RTC);
  Wire.send(0xc); // select register
  Wire.send(halt);
  Wire.endTransmission();
}

// **************************************************************
// set date. Note year is 2 digits
// **************************************************************
void BV4236::setDate(int year, int month, int day)
{
  Wire.beginTransmission(I2RTC);
  Wire.send(0x5); // start at day of month
  Wire.send(dec2bcd(day)); 
  Wire.send(dec2bcd(month)); 
  Wire.send(dec2bcd(year)); 
  Wire.endTransmission();
    
}

// **************************************************************
// set time. Note year is 2 digits
// This will also start the clock
// **************************************************************
void BV4236::setTime(int hours, int minutes, int seconds)
{
  Wire.beginTransmission(I2RTC);
  Wire.send(0x1); // start at seconds
  Wire.send(dec2bcd(seconds)); 
  Wire.send(dec2bcd(minutes)); 
  Wire.send(dec2bcd(hours)); 
  Wire.endTransmission();
    
}

// **************************************************************
// set day of week
// **************************************************************
void BV4236::setDOW(int dow)
{
  Wire.beginTransmission(I2RTC);
  Wire.send(0x4); // start at dow
  Wire.send(dec2bcd(dow)); 
  Wire.endTransmission();    
}

