/*
  BV4627 8 way multi-interface relay board from ByVac
  Copyright (c) 2011 Jim Spence.  All right reserved.
  www.byvac.com - see terms and conditions for using hardware
  
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/  

#include "WProgram.h"
#include "bv4627.h"
 
BV4627::BV4627(char i2adr)
{
  _i2rlyadr=i2adr;  
  Wire.begin(); // join i2c bus 
}

// ==============================================================
// I2C utilities
// ==============================================================
// ==============================================================
// **************************************************************
// gets a 16 bit value from the i2c bus
// **************************************************************
int BV4627::i2_16bit()
{
int rv;
  Wire.requestFrom(_i2rlyadr, 2); // returns 2 bytes
  rv=Wire.receive()*256; // high byte
  rv+=Wire.receive(); // low byte
  return rv;
}

// ==============================================================
// I2C 8 WAY RELAY
// ==============================================================
// ==============================================================

// **************************************************************
// Switches an individual relay after a given time, 
// relay A=1, H=8
// **************************************************************
void BV4627::click(char rly, char on, int del)
{
  if(!del) del=1; // 0 is not allowed
  
  if(rly <1) rly = 1;
  if(rly>8) rly = 8;
  
  rly+=9; // relays range 10 to 18
  
  // format is rly, on/off, high_del, low_del
  Wire.beginTransmission(_i2rlyadr);
  Wire.send(rly);  // relay comand
  Wire.send(on);  // on or off
  Wire.send((del>>8)&0xff); // high byte
  Wire.send(del & 0xff); // low byte
  Wire.endTransmission();
} 

// **************************************************************
// gets the value of an individual relay timer
// **************************************************************
int BV4627::timer(char rly)
{
  Wire.beginTransmission(_i2rlyadr);
  Wire.send(30);  // relay comand to get timer value
  Wire.send(rly); // the timer concerned
  Wire.endTransmission();
  // The device will now send data
  return i2_16bit();
}

// **************************************************************
// turns all relays off
// **************************************************************
void BV4627::off()
{
  Wire.beginTransmission(_i2rlyadr);
  Wire.send(18);  // off comand
  Wire.endTransmission();
}

// **************************************************************
// turns all relays on
// **************************************************************
void BV4627::on()
{
  Wire.beginTransmission(_i2rlyadr);
  Wire.send(19);  // on comand
  Wire.endTransmission();
}

// **************************************************************
// Binary on - switches relays in accord with the individual bits
// set in param rly. LSB is relay A
// **************************************************************
void BV4627::bin(char rly)
{
  Wire.beginTransmission(_i2rlyadr);
  Wire.send(20);  // bin comand
  Wire.send(rly);
  Wire.endTransmission();
}

// **************************************************************
// sets I2C address - note will not respond to old I2C address
// after a reset
// **************************************************************
void BV4627::setaddress(char newAddress)
{
  Wire.beginTransmission(_i2rlyadr);
  Wire.send(82);  // new address comand
  Wire.send(newAddress);
  Wire.endTransmission();
}

// **************************************************************
// get device id, this should be 4627
// **************************************************************
int BV4627::deviceid()
{
  Wire.beginTransmission(_i2rlyadr);
  Wire.send(83);  // device id comand
  Wire.endTransmission();
  return i2_16bit();
}

// **************************************************************
// get device firmware version, this is a string!!
// **************************************************************
void BV4627::version(char *ver)
{
char b, tmp[20];  
  Wire.beginTransmission(_i2rlyadr);
  Wire.send(84);  // firmware comand
  Wire.endTransmission();
  // return as a string in the format x.x
  Wire.requestFrom(_i2rlyadr, 2); // returns 2 bytes
  b=Wire.receive(); // major byte
  itoa(b,tmp,10); // convert to string
  *ver=0; // just in case
  strcpy(ver,tmp);
  strcat(ver,".");
  b=Wire.receive(); // minor byte
  itoa(b,tmp,10);
  strcat(ver,tmp);
}

// **************************************************************
// reset device
// **************************************************************
void BV4627::reset()
{
  Wire.beginTransmission(_i2rlyadr);
  Wire.send(85);  // reset comand
  Wire.endTransmission();
}
