/*
  BV4618_S I2C and (serial) charatcer display controller  ByVac
  Copyright (c) 2011 Jim Spence.  All right reserved.
  www.byvac.com - see terms and conditions for using hardware
  
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/  


#include "WConstants.h"
#include "vt100.h"

// **************************************************************
// constructor
// **************************************************************
VT100::VT100(uint8_t receivePin, uint8_t transmitPin)
{
    _receivePin = receivePin;
    _transmitPin = transmitPin;
}

// **************************************************************
// initialises display
// **************************************************************
void VT100::begin(long speed)
{
    _baudRate = speed;
    _bitPeriod = 1000000 / _baudRate;
    digitalWrite(_transmitPin, HIGH);
    pinMode(_transmitPin, OUTPUT);
    pinMode(_receivePin, INPUT);
    delay(1); // steady state high to establish
    putch(13);
    delay(200);
}

// ******************************************************************************
// clears buffer
// ******************************************************************************
void clear_tib()
{
	// buffer reset
	cib=0;
	Pptr=Gptr=0;  // pointer initialise
	bfullF=0;	  // buffer not full
}

// ******************************************************************************
// input in serial interrupt, this fills a buffer up using pre-incrementing
// ******************************************************************************
void fill_buf()
{
	
	if((Pptr+1) == Gptr) bfullF=1;
    // full when trying to wrap on top of Gptr
    if(((Pptr+1) > TIB_MAX) && Gptr==0) bfullF=1;

    // if buffer full then stop reception
    // next character is in the hardware buffer
    if(bfullF) {
        nRTS=1;
        return;
    }
        
    // increment and check for wrap
    if(++Pptr > TIB_MAX) Pptr=0;

    // get and place char, also clears IR
    TIB[Pptr]=RCREG;
    cib++; // easiest way to keep track
}
			
// ******************************************************************************
// removes a char from the buffer if there is one, if not it will wait here
// ******************************************************************************
unsigned char getch (void)
{
unsigned char c;
static unsigned char t=1;	// cursor toggle

	// wait until there is a char
	while (!cib) {		// Wait for character
		if((wait % CURFL)==0) { 
			lcd_cFlash(t);
			t^=1;  // toggle for next time
		}	
	}
	
	// if buffer is full then the last char is actually in the h/w buffer
    if(bfullF) c=RCREG; // also clears interrupt
    else {
    	if(++Gptr > TIB_MAX) Gptr=0;
        c = TIB[Gptr]; 
    }
    // make sure recieve is enabled, buffer not full so room for more
    nRTS=0; // enable recieve
    --cib;  // redune number of char in buffer
    bfullF=0; // cant be full now
    return c;	
}

unsigned char getche (void)
{
	// Read character and echo back
	char c;
	putch (c = getch ());

	return c;
}

void putch (char c)
{
	//Put the specified character on the serial port
	while (!TXIF);		// Wait for send buffer to empty
	TXREG = c;			// Send character
}

bit kbhit (void)
{
	// Return true if a character is waiting on the serial port
	if(cib) return 1;
	else return 0;
}


// **************************************************************
// Original code from SoftwareSerial
// Buffered input
// **************************************************************
int VT100::read()
{
  int val = 0;
  int bitDelay = _bitPeriod - clockCyclesToMicroseconds(50);
  
  // one byte of serial data (LSB first)
  // ...--\    /--\/--\/--\/--\/--\/--\/--\/--\/--...
  //	 \--/\--/\--/\--/\--/\--/\--/\--/\--/
  //	start  0   1   2   3   4   5   6   7 stop

  while (digitalRead(_receivePin));

  // confirm that this is a real start bit, not line noise
  if (digitalRead(_receivePin) == LOW) {
    // frame start indicated by a falling edge and low start bit
    // jump to the middle of the low start bit
    delayMicroseconds(bitDelay / 2 - clockCyclesToMicroseconds(50));
	
    // offset of the bit in the byte: from 0 (LSB) to 7 (MSB)
    for (int offset = 0; offset < 8; offset++) {
	// jump to middle of next bit
	delayMicroseconds(bitDelay);
	
	// read bit
	val |= digitalRead(_receivePin) << offset;
    }
	
    delayMicroseconds(_bitPeriod);
    
    return val;
  }
  
  return -1;
}

// **************************************************************
// from SoftwareSerial
// **************************************************************
void VT100::putch(uint8_t b)
{
  int bitDelay = _bitPeriod - clockCyclesToMicroseconds(50); // a digitalWrite is about 50 cycles
  byte mask;

  digitalWrite(_transmitPin, LOW);
  delayMicroseconds(bitDelay);

  for (mask = 0x01; mask; mask <<= 1) {
    if (b & mask){ // choose bit
      digitalWrite(_transmitPin,HIGH); // send 1
    }
    else{
      digitalWrite(_transmitPin,LOW); // send 1
    }
    delayMicroseconds(bitDelay);
  }

  digitalWrite(_transmitPin, HIGH);
  delayMicroseconds(bitDelay);
}

// **************************************************************
// put sting to display with escape codes
// **************************************************************
void VT100::puts(char *s)
{
    while(*s) 
        putch(*(s++));
}

// Private Methods /////////////////////////////////////////////////////////////

void VT100::printNumber(unsigned long n, uint8_t base)
{
  unsigned char buf[8 * sizeof(long)]; // Assumes 8-bit chars. 
  unsigned long i = 0;

  if (n == 0) {
    putch('0');
    return;
  } 

  while (n > 0) {
    buf[i++] = n % base;
    n /= base;
  }

  for (; i > 0; i--)
    putch((char) (buf[i - 1] < 10 ? '0' + buf[i - 1] : 'A' + buf[i - 1] - 10));
}
